﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Tutorial_Web_Applikation.Models;
using Tutorial_Web_Applikation.Services;

namespace Tutorial_Web_Applikation.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class AuthController : Controller
    {
        private ShopContext _context;
        private readonly IPasswordHash _passwordHashService;
        private readonly ITokenGenerator _tokenService;

        public AuthController(ShopContext context, IPasswordHash passwordHashService, ITokenGenerator tokenService)
        {
            _context = context;
            _passwordHashService = passwordHashService;
            _tokenService = tokenService;
        }

        [HttpPost("register")]
        public async Task<ActionResult<User>> Register(UserRequest request)
        {
            var exisitingUser = await _context.Users!.FirstOrDefaultAsync(user => user.Username == request.Username);

            if (exisitingUser != null)
            {
                return BadRequest("Username already exists");
            }
            else
            {
                _passwordHashService.HashPassword(request.Password!, out byte[] passwordHash, out byte[] passwordSalt);

                User user = new User();
                user.Username = request.Username;
                user.PasswordHash = passwordHash;
                user.PasswordSalt = passwordSalt;
                user.Role = request.Role;
                _context.Users!.Add(user);
                await _context.SaveChangesAsync();
                return Ok(user);

            }
        }
        [HttpPost("signin")]
        public async Task<IActionResult> Signin(UserLoginRequest request)
        {

            var exisitingUser = await _context.Users!.FirstOrDefaultAsync(user => user.Username == request.Username);

            if (exisitingUser == null)
            {
                return BadRequest("User not found");
            }
            else if (!_passwordHashService.VerifyPasssword(request.Password, exisitingUser.PasswordHash, exisitingUser.PasswordSalt))
            {
                return BadRequest("Incorrect password");
            }
            else
            {

                var accessToken = _tokenService.GenerateAccessToken(exisitingUser);
                var refreshToken = _tokenService.GenerateRefreshToken();
                await _tokenService.SetRefreshToken(refreshToken, Response);

                exisitingUser.RefreshToken = refreshToken.Token;
                exisitingUser.DateCreated = refreshToken.Created;
                exisitingUser.TokenExpires = refreshToken.Expires;

                await _context.SaveChangesAsync();
                return Ok(accessToken);
            }

        }


        [HttpGet("protected")]
        [Authorize]
        public string ProtectedAPI()
        {

            return "Protected Resource.You are authorized to access this resource";
        }

        [HttpPost("refresh")]
        public async Task<ActionResult<string>> RefreshToken()
        {
            var refreshToken = Request.Cookies["refreshToken"];

            var existinguser = await _context.Users!.FirstOrDefaultAsync(user => user.RefreshToken == refreshToken);


            if (existinguser == null)
            {
                return Unauthorized("Invalid Refresh Token.");
            }
            else if (existinguser.TokenExpires < DateTime.Now)
            {
                return Unauthorized("Token Expired");
            }
            else
            {
                string accessToken = _tokenService.GenerateAccessToken(existinguser);
                var newRefreshToken = _tokenService.GenerateRefreshToken();
                await _tokenService.SetRefreshToken(newRefreshToken, Response);
                return Ok(accessToken);
            }
        }
    }
}
