﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Tutorial_Web_Applikation.Models;
using Swashbuckle.AspNetCore.Annotations;
using Microsoft.AspNetCore.Cors;
using Microsoft.AspNetCore.Authorization;

namespace Tutorial_Web_Applikation.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ProductsController : ControllerBase
    {
        private readonly ShopContext _context;

        public ProductsController(ShopContext _context)
        {
            this._context = _context;
        }

        /// <summary>
        /// Get all products
        /// </summary>
        /// <returns>Returns all Products in the Product Table with all Data they have</returns>
        [HttpGet]
        public ActionResult GetAllProducts()
        {
            var _products = _context.Products.ToList();
            return Ok(_products);
        }

        /// <summary>
        /// Get a product by id
        /// </summary>
        /// <param name="_id">The ID of the desired product</param>
        /// <returns>Returns the product that has the corresponding ID</returns>
        [HttpGet("{id}")]
        [EnableCors("AllowBbbOrigin")]
        public ActionResult GetProduct(int _id)
        {
            var _product = _context.Products.Find(_id);

            if (_product == null)
            {
                return NotFound();
            }

            return Ok(_product);
        }

        /// <summary>
        /// Create a new product
        /// </summary>
        /// <param name="product">The Product in JSON-Format that should be inserted</param>
        /// <returns>Returns only the Status</returns>
        [HttpPost]
        [Authorize]
        public async Task<ActionResult> PostProduct(Product product)
        {
            _context.Products.Add(product);
            await _context.SaveChangesAsync();

            return CreatedAtAction(
                nameof(GetProduct), 
                new { id = product.Id }, 
                product);
        }

        /// <summary>
        /// Update a product
        /// </summary>
        /// <param name="id">The ID of the product that should be updated</param>
        /// <param name="product">The new product with the changes</param>
        /// <returns>Returns only the Status</returns>
        [HttpPut("{id}")]
        [Authorize]
        public async Task<ActionResult> PutProduct(int id, [FromBody] Product product)
        {
            _context.Entry(product).State = EntityState.Modified;

            try
            {
                await _context.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!_context.Products.Any(_p => _p.Id == id))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return NoContent();
        }

        /// <summary>
        /// Delete a product
        /// </summary>
        /// <param name="id">The ID of the product that should be deleted</param>
        /// <returns>Returns the deleted product</returns>
        [HttpDelete("{id}")]
        [Authorize]
        public async Task<ActionResult> DeleteProduct(int id)
        {
            var product = await _context.Products.FindAsync(id);

            if (product == null ) 
            { 
                return NotFound(); 
            }

            _context.Products.Remove(product);
            await _context.SaveChangesAsync();

            return Ok(product);
        }
    }
}
