
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.EntityFrameworkCore;
using Microsoft.Identity.Client;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using System.Text;
using System.Threading.Tasks;
using Tutorial_Web_Applikation.Models;
using Tutorial_Web_Applikation.Services;

namespace Tutorial_Web_Applikation
{
    public class Program
    {
        public string name { get; set; } = "Test";

#pragma warning disable IDE1006 // Naming Styles
        public static async Task Main(string[] args)
#pragma warning restore IDE1006 // Naming Styles
        {
            var _builder = WebApplication.CreateBuilder(args);

            // Add services to the container.

            _builder.Services.AddControllers();
            // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
            _builder.Services.AddEndpointsApiExplorer();

            _builder.Services.AddSwaggerGen(_c =>
            {
                _c.SwaggerDoc("v1", new OpenApiInfo { Title = "Tutorial Web Applikation", Version = "v1" });

                var _xmlFile = Path.Combine(AppContext.BaseDirectory, "Tutorial Web Applikation.xml");
                _c.IncludeXmlComments(_xmlFile);
            });



            _builder.Services.AddDbContext<ShopContext>(_options =>
            {
                _options.UseSqlServer(_builder.Configuration.GetConnectionString("DefaultConnection"));
            });


            _builder.Services.AddCors(_options =>
            {
                _options.AddPolicy("AllowBbbOrigin",
                    _policy =>
                    {
                        _policy.WithOrigins("https://bbbaden.ch",
                                           "https://www.bbbaden.ch",
                                           "https://localhost:7032")
                              .AllowAnyHeader()
                              .AllowAnyMethod();
                    });
                _options.AddPolicy("AllowLocalhostOrigin",
                    _policy =>
                    {
                        _policy.WithOrigins("https://localhost:7032")
                              .WithMethods("GET", "POST", "PUT", "DELETE")
                              .AllowAnyHeader();
                    });
            });

            //Service for Hashing and verifying password 
            _builder.Services.AddScoped<IPasswordHash, PasswordHashService>();
            _builder.Services.AddScoped<ITokenGenerator, TokenGeneratorService>();

            //Authentication service
            _builder.Services.AddAuthentication(_options =>
            {
                _options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
                _options.DefaultScheme = JwtBearerDefaults.AuthenticationScheme;
                _options.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
            }).AddJwtBearer(_options =>
            {// JwtConfig:Key ist der symetrische Schlssel fr die Verschlsselung
                var _key = _builder.Configuration.GetSection("JwtConfig:Key").Value;
                _options.TokenValidationParameters = new TokenValidationParameters
                {

                    ValidateIssuerSigningKey = true,
                    IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(_key)),
                    ValidateIssuer = false, // Optional: Validierung des Issuer
                    ValidateAudience = false,
                    ValidateLifetime = true, // Zeitstempel berprfen
                    ClockSkew = TimeSpan.Zero // Optional: Pufferzeit deaktivieren  siehe Zustzliche Angaben zum Auftrag
                };
            });


            var _app = _builder.Build();

            // Configure the HTTP request pipeline.
            if (_app.Environment.IsDevelopment())
            {
                _app.UseSwagger();
                _app.UseSwaggerUI();
            }
            else
            {
                _app.UseHsts();
            }

            using (var _scope = _app.Services.CreateScope())
            {
                var _dbContext = _scope.ServiceProvider.GetRequiredService<ShopContext>();
                _dbContext.Database.Migrate(); // Apply all migrations
            }


            _app.UseHttpsRedirection();

            _app.UseAuthorization();

            _app.MapControllers();
            
            _app.UseCors("AllowLocalhostOrigin");

            _app.Run();
        }
    }
}
